@@ -1,53 +1,56 @@
-/*
-
-Copyright (C) 2003 David Bateman
-
-This program is free software; you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
-
-This program is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; see the file COPYING.  If not, see
-<http://www.gnu.org/licenses/>.
-
-In addition to the terms of the GPL, you are permitted to link
-this program with any Open Source program, as defined by the
-Open Source Initiative (www.opensource.org)
-
-*/
+//Copyright (C) 2003 David Bateman
+//
+// This program is free software; you can redistribute it and/or
+// modify it under the terms of the GNU General Public License as
+// published by the Free Software Foundation; either version 3 of the
+// License, or (at your option) any later version.
+//
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+// General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, see
+// <http://www.gnu.org/licenses/>.
+//
+// In addition to the terms of the GPL, you are permitted to link this
+// program with any Open Source program, as defined by the Open Source
+// Initiative (www.opensource.org)
 
 #include <iostream>
+
 #include <octave/config.h>
-#include <octave/unwind-prot.h>
+#include <octave/byte-swap.h>
+#include <octave/gripes.h>
+#include <octave/lo-ieee.h>
+#include <octave/oct-hdf5.h>
+#include <octave/oct-locbuf.h>
+#include <octave/oct-obj.h>
+#include <octave/ov.h>
 #include <octave/pr-output.h>
+
+#include <octave/ls-oct-ascii.h>
+
 #include "galois.h"
 #include "ov-galois.h"
 
-#if ! defined (HAVE_OCTAVE_29) && ! defined (HAVE_OCTAVE_30)
-extern int Vstruct_levels_to_print;
+#if defined (DEFINE_OCTAVE_ALLOCATOR)
+DEFINE_OCTAVE_ALLOCATOR (octave_galois);
 #endif
 
-#include <octave/byte-swap.h>
-#include <octave/ls-oct-ascii.h>
+DEFINE_OV_TYPEID_FUNCTIONS_AND_DATA (octave_galois, "galois", "galois");
 
-DEFINE_OCTAVE_ALLOCATOR(octave_galois);
-DEFINE_OV_TYPEID_FUNCTIONS_AND_DATA(octave_galois, "galois", "galois");
-
-octave_value octave_galois::resize (const dim_vector& dv, bool) const
-{ 
-  if (dv.length() > 2)
+octave_value
+octave_galois::resize (const dim_vector& dv, bool) const
+{
+  if (dv.length () > 2)
     {
       error ("Can not resize galois structure to NDArray");
       return octave_value ();
     }
-  galois retval (gval); 
-  retval.resize (dv(0), dv(1)); 
+  galois retval (gval);
+  retval.resize (dv);
   return new octave_galois (retval);
 }
 
@@ -61,44 +64,48 @@ octave_galois::dotref (const octave_value_list& idx)
   std::string nm = idx(0).string_value ();
 
   if (nm == __GALOIS_PRIMPOLY_STR)
-    retval(0) = octave_value ((double)gval.primpoly());
+    retval(0) = octave_value ((double)gval.primpoly ());
   else if (nm == __GALOIS_ORDER_STR)
-    retval(0) = octave_value ((double)gval.m());
-  else if (nm == __GALOIS_DATA_STR) {
-    int r = gval.rows();
-    int c = gval.columns();
-    Matrix data(r,c);
-    for (int i=0; i< r; i++)
-      for (int j=0; j< c; j++)
-	data(i,j) = (double)gval(i,j);
-    retval(0) = octave_value (data);
-  } 
+    retval(0) = octave_value ((double)gval.m ());
+  else if (nm == __GALOIS_DATA_STR)
+    {
+      int r = gval.rows ();
+      int c = gval.columns ();
+      Matrix data (r, c);
+      for (int i = 0; i < r; i++)
+        for (int j = 0; j < c; j++)
+          data(i, j) = (double)gval(i, j);
+      retval(0) = octave_value (data);
+    }
 #ifdef GALOIS_DISP_PRIVATES
   else if (nm == __GALOIS_LENGTH_STR)
-    retval(0) = octave_value((double)gval.n());
-  else if (nm == __GALOIS_ALPHA_TO_STR) {
-    int n = gval.n();
-    Matrix data(n+1,1);
-    for (int i=0; i< n+1; i++)
-      data(i,0) = (double)gval.alpha_to(i);
-    retval(0) = octave_value (data);
-  } else if (nm == __GALOIS_INDEX_OF_STR) {
-    int n = gval.n();
-    Matrix data(n+1,1);
-    for (int i=0; i< n+1; i++)
-      data(i,0) = (double)gval.index_of(i);
-    retval(0) = octave_value (data);
-  }
+    retval(0) = octave_value ((double)gval.n ());
+  else if (nm == __GALOIS_ALPHA_TO_STR)
+    {
+      int n = gval.n ();
+      Matrix data (n+1, 1);
+      for (int i = 0; i < n+1; i++)
+        data(i, 0) = (double)gval.alpha_to (i);
+      retval(0) = octave_value (data);
+    }
+  else if (nm == __GALOIS_INDEX_OF_STR)
+    {
+      int n = gval.n ();
+      Matrix data (n+1, 1);
+      for (int i = 0; i < n+1; i++)
+        data(i, 0) = (double)gval.index_of (i);
+      retval(0) = octave_value (data);
+    }
 #endif
   else
-    error ("galois structure has no member `%s'", nm.c_str ());    
+    error ("galois structure has no member `%s'", nm.c_str ());
 
   return retval;
 }
 
 octave_value
 octave_galois::do_index_op (const octave_value_list& idx,
-			    bool resize_ok)
+                            bool resize_ok)
 {
   octave_value retval;
 
@@ -108,27 +115,27 @@ octave_galois::do_index_op (const octave_value_list& idx,
     {
     case 2:
       {
-	idx_vector i = idx (0).index_vector ();
-	idx_vector j = idx (1).index_vector ();
-  
-	retval = new octave_galois (gval.index (i, j, resize_ok));
+        idx_vector i = idx (0).index_vector ();
+        idx_vector j = idx (1).index_vector ();
+
+        retval = new octave_galois (gval.index (i, j, resize_ok));
       }
       break;
 
     case 1:
       {
-	idx_vector i = idx (0).index_vector ();
+        idx_vector i = idx (0).index_vector ();
 
-	retval = new octave_galois (gval.index (i, resize_ok));
+        retval = new octave_galois (gval.index (i, resize_ok));
       }
       break;
 
     default:
       {
-	std::string n = type_name ();
+        std::string n = type_name ();
 
-	error ("invalid number of indices (%d) for %s value",
-	       len, n.c_str ());
+        error ("invalid number of indices (%d) for %s value",
+               len, n.c_str ());
       }
       break;
     }
@@ -138,7 +145,7 @@ octave_galois::do_index_op (const octave_value_list& idx,
 
 octave_value
 octave_galois::subsref (const std::string &type,
-			const std::list<octave_value_list>& idx)
+                        const std::list<octave_value_list>& idx)
 {
   octave_value retval;
 
@@ -152,14 +159,14 @@ octave_galois::subsref (const std::string &type,
 
     case '.':
       {
-	octave_value_list t = dotref (idx.front ());
+        octave_value_list t = dotref (idx.front ());
 
-	retval = (t.length () == 1) ? t(0) : octave_value (t);
+        retval = (t.length () == 1) ? t(0) : octave_value (t);
       }
       break;
 
     case '{':
-      error ("%s cannot be indexed with %c", type_name().c_str(), type[0]);
+      error ("%s cannot be indexed with %c", type_name ().c_str (), type[0]);
       break;
 
     default:
@@ -181,7 +188,7 @@ octave_galois::is_true (void) const
     {
       boolMatrix m = (gval.all () . all ());
 
-      retval = (m.rows () == 1 && m.columns () == 1 && m(0,0));
+      retval = (m.rows () == 1 && m.columns () == 1 && m(0, 0));
     }
 
   return retval;
@@ -193,11 +200,15 @@ octave_galois::print_as_scalar (void) const
   int nr = rows ();
   int nc = columns ();
 
-  return (nr == 1 && nc == 1 || (nr == 0 || nc == 0));
+  return ((nr == 1 && nc == 1) || (nr == 0 || nc == 0));
 }
 
 void
+#if defined (HAVE_OCTAVE_BASE_VALUE_PRINT_CONST)
 octave_galois::print (std::ostream& os, bool) const
+#else
+octave_galois::print (std::ostream& os, bool)
+#endif
 {
   print_raw (os);
 }
@@ -205,43 +216,49 @@ octave_galois::print (std::ostream& os, bool) const
 void
 octave_galois::print_raw (std::ostream& os, bool) const
 {
-  unwind_protect::begin_frame ("octave_galois_print");
-
-  unwind_protect_int (Vstruct_levels_to_print);
-
   bool first = true;
-  int m = gval.m();
-  int primpoly = gval.primpoly();
-  Matrix data(gval.rows(), gval.cols());
+  int m = gval.m ();
+  int primpoly = gval.primpoly ();
+  Matrix data (gval.rows (), gval.cols ());
 
   indent (os);
 
   if (m == 1)
     os << "GF(2) array.";
-  else {
-    os << "GF(2^" << m << ") array. Primitive Polynomial = ";
+  else
+    {
+      os << "GF(2^" << m << ") array. Primitive Polynomial = ";
 
-    for (int i=m; i>=0; i--) {
-      if (primpoly & (1<<i)) {
-	if (i > 0) {
-	  if (first) {
-	    first = false;
-	    os << "D";
-	  } else
-	    os << "+D";
-	  if (i != 1)
-	    os << "^" << i;
-	} else {
-	  if (first) {
-	    first = false;
-	    os << "1";
-	  } else
-	    os << "+1";
-	}
-      }
+      for (int i = m; i >= 0; i--)
+        {
+          if (primpoly & (1<<i))
+            {
+              if (i > 0)
+                {
+                  if (first)
+                    {
+                      first = false;
+                      os << "D";
+                    }
+                  else
+                    os << "+D";
+                  if (i != 1)
+                    os << "^" << i;
+                }
+              else
+                {
+                  if (first)
+                    {
+                      first = false;
+                      os << "1";
+                    }
+                  else
+                    os << "+1";
+                }
+            }
+        }
+      os << " (decimal " << primpoly << ")";
     }
-    os << " (decimal " << primpoly << ")";
-  }
   newline (os);
   newline (os);
   indent (os);
@@ -250,14 +267,12 @@ octave_galois::print_raw (std::ostream& os, bool) const
   newline (os);
   newline (os);
 
-  for (int i = 0; i < gval.rows(); i++)
-    for (int j = 0; j < gval.columns(); j++)
-      data(i,j) = (double)gval(i,j);
+  for (int i = 0; i < gval.rows (); i++)
+    for (int j = 0; j < gval.columns (); j++)
+      data(i, j) = (double)gval(i, j);
 
   octave_print_internal (os, data, false, current_print_indent_level ());
   newline (os);
-
-  unwind_protect::run_frame ("octave_galois_print");
 }
 
 bool
@@ -267,7 +282,13 @@ octave_galois::print_name_tag (std::ostream& os, const std::string& name) const
 
   indent (os);
 
+  // Vstruct_levels_to_print was made static in Octave 3.4, which means
+  // the name_tag might be printed on a seperate line when it shouldn't be.
+#if 0
   if (Vstruct_levels_to_print < 0)
+#else
+  if (false)
+#endif
     os << name << " = ";
   else
     {
@@ -293,7 +314,7 @@ octave_galois::double_value (bool) const
   if (rows () > 0 && columns () > 0)
     {
       gripe_implicit_conversion ("Octave:array-as-scalar",
-				 "real matrix", "real scalar");
+                                 "real matrix", "real scalar");
 
       retval = (double) gval (0, 0);
     }
@@ -313,7 +334,7 @@ octave_galois::complex_value (bool) const
   if (rows () > 0 && columns () > 0)
     {
       gripe_implicit_conversion ("Octave:array-as-scalar",
-				 "real matrix", "real scalar");
+                                 "real matrix", "real scalar");
 
       retval = (double) gval (0, 0);
     }
@@ -328,10 +349,10 @@ octave_galois::matrix_value (bool) const
 {
   Matrix retval;
 
-  retval.resize(rows(),columns());
-  for (int i=0; i<rows(); i++)
-    for (int j=0; j<columns(); j++)
-      retval(i,j) = gval(i,j);
+  retval.resize (rows (), columns ());
+  for (int i = 0; i < rows (); i++)
+    for (int j = 0; j < columns (); j++)
+      retval(i, j) = gval(i, j);
 
   return retval;
 }
@@ -341,34 +362,36 @@ octave_galois::array_value (bool) const
 {
   int nr = rows ();
   int nc = columns ();
-  dim_vector dv(nr,nc);
+  dim_vector dv (nr, nc);
   NDArray retval (dv);
 
-  for (int i=0; i<nr; i++)
-    for (int j=0; j<nc; j++)
-      retval(i + j*nr) = gval(i,j);
+  for (int i = 0; i < nr; i++)
+    for (int j = 0; j < nc; j++)
+      retval(i + j*nr) = gval(i, j);
 
   return retval;
 }
 
 void
 octave_galois::assign (const octave_value_list& idx,
-			       const galois& rhs)
+                       const galois& rhs)
 {
   int len = idx.length ();
 
-  if (gval.have_field() && rhs.have_field()) {
-    if ((gval.m() != rhs.m()) || (gval.primpoly() != rhs.primpoly())) {
-      (*current_liboctave_error_handler) ("can not assign data between two different Galois Fields");
-      return;
+  if (gval.have_field () && rhs.have_field ())
+    {
+      if ((gval.m () != rhs.m ()) || (gval.primpoly () != rhs.primpoly ()))
+        {
+          (*current_liboctave_error_handler) ("can not assign data between two different Galois Fields");
+          return;
+        }
     }
-  }
 
   switch (len)
     {
     case 2:
       {
-	idx_vector i = idx (0).index_vector ();
+        idx_vector i = idx (0).index_vector ();
 
         if (! error_state)
           {
@@ -382,21 +405,21 @@ octave_galois::assign (const octave_value_list& idx,
 
     case 1:
       {
-	idx_vector i = idx (0).index_vector ();
+        idx_vector i = idx (0).index_vector ();
 
-	if (! error_state)
-	  gval.assign (i, rhs);
+        if (! error_state)
+          gval.assign (i, rhs);
       }
       break;
 
     default:
       error ("invalid number of indices (%d) for galois assignment",
-	     len);
+             len);
       break;
     }
 }
 
-bool 
+bool
 octave_galois::save_ascii (std::ostream& os)
 {
   dim_vector d = dims ();
@@ -408,14 +431,14 @@ octave_galois::save_ascii (std::ostream& os)
   os << "# prim: " << primpoly () << "\n";
   os << "# ndims: " << d.length () << "\n";
 
-  for (int i=0; i < d.length (); i++)
+  for (int i = 0; i < d.length (); i++)
     os << " " << d (i);
 
   os << "\n" << tmp;
   return true;
 }
 
-bool 
+bool
 octave_galois::load_ascii (std::istream& is)
 {
   int mord, prim, mdims;
@@ -428,37 +451,37 @@ octave_galois::load_ascii (std::istream& is)
       dv.resize (mdims);
 
       for (int i = 0; i < mdims; i++)
-	is >> dv(i);
+        is >> dv(i);
 
-      if (dv.length() != 2)
-	{
-	  error ("load: N-D galois matrix not supported");
-	  success = false;
-	}
+      if (dv.length () != 2)
+        {
+          error ("load: N-D galois matrix not supported");
+          success = false;
+        }
       else
-	{
+        {
 
-	  Matrix tmp (dv(0), dv(1));
-	  is >> tmp;
+          Matrix tmp (dv(0), dv(1));
+          is >> tmp;
 
-	  if (!is) 
-	    {
-	      error ("load: failed to load matrix constant");
-	      success = false;
-	    }
-	  gval = galois (tmp, mord, prim);
-	}
-    }
-  else 
-    {
-      error ("load: failed to extract galois field order, primitive and/or imension");
-      success = false;
+          if (!is)
+            {
+              error ("load: failed to load matrix constant");
+              success = false;
+            }
+          gval = galois (tmp, mord, prim);
+        }
     }
+  else
+  {
+    error ("load: failed to extract galois field order, primitive and/or imension");
+    success = false;
+  }
 
   return success;;
 }
 
-bool 
+bool
 octave_galois::save_binary (std::ostream& os, bool& save_as_floats)
 {
   char tmp = m ();
@@ -469,13 +492,13 @@ octave_galois::save_binary (std::ostream& os, bool& save_as_floats)
   dim_vector d = dims ();
 
   // Don't handle N-D arrays yet
-  if (d.length() != 2)
+  if (d.length () != 2)
     return false;
 
   // Use negative value for ndims to be consistent with other formats
-  itmp = - d.length();
+  itmp = - d.length ();
   os.write (X_CAST (char *, &itmp), 4);
-  for (int i=0; i < d.length (); i++)
+  for (int i = 0; i < d.length (); i++)
     {
       itmp = d(i);
       os.write (X_CAST (char *, &itmp), 4);
@@ -495,9 +518,9 @@ octave_galois::save_binary (std::ostream& os, bool& save_as_floats)
   return true;
 }
 
-bool 
+bool
 octave_galois::load_binary (std::istream& is, bool swap,
-				 oct_mach_info::float_format fmt)
+                            oct_mach_info::float_format fmt)
 {
   char mord;
   int32_t prim, mdims;
@@ -524,23 +547,23 @@ octave_galois::load_binary (std::istream& is, bool swap,
       dv.resize (mdims);
 
       for (int i = 0; i < mdims; i++)
-	{
-	  if (! is.read (X_CAST (char *, &di), 4))
-	    return false;
-	  if (swap)
-	    swap_bytes <4> (X_CAST (char *, &di));
-	  dv(i) = di;
-	}
+        {
+          if (! is.read (X_CAST (char *, &di), 4))
+            return false;
+          if (swap)
+            swap_bytes <4> (X_CAST (char *, &di));
+          dv(i) = di;
+        }
 
       char tmp;
       if (! is.read (X_CAST (char *, &tmp), 1))
-	return false;
+        return false;
 
       Matrix m (dv(0), dv(1));
       double *re = m.fortran_vec ();
       read_doubles (is, re, X_CAST (save_type, tmp), dv.numel (), swap, fmt);
       if (error_state || ! is)
-	return false;
+        return false;
 
       gval = galois (m, mord, prim);
     }
@@ -548,75 +571,91 @@ octave_galois::load_binary (std::istream& is, bool swap,
   return true;
 }
 
-#if defined (HAVE_HDF5)
 bool
-octave_galois::save_hdf5 (hid_t loc_id, const char *name, bool save_as_floats)
+octave_galois::save_hdf5 (octave_hdf5_id loc_id, const char *name, bool save_as_floats)
 {
+  bool retval = true;
+
+#if defined (HAVE_HDF5)
+
   Matrix mval = matrix_value ();
   hid_t group_hid = -1;
+#if HAVE_HDF5_18
+  group_hid = H5Gcreate (loc_id, name, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+#else
   group_hid = H5Gcreate (loc_id, name, 0);
+#endif
   if (group_hid < 0 ) return false;
 
   dim_vector d = dims ();
-  OCTAVE_LOCAL_BUFFER(hsize_t, hdims, d.length () > 2 ? d.length () : 3);
+  OCTAVE_LOCAL_BUFFER (hsize_t, hdims, d.length () > 2 ? d.length () : 3);
   hid_t space_hid = -1, data_hid = -1;
-  bool retval = true;
   char tmp;
   int32_t itmp;
 
   space_hid = H5Screate_simple (0, hdims, (hsize_t*) 0);
-  if (space_hid < 0) 
+  if (space_hid < 0)
     {
       H5Gclose (group_hid);
       return false;
     }
 
-  data_hid = H5Dcreate (group_hid, "m", H5T_NATIVE_UCHAR, space_hid, 
-			H5P_DEFAULT);
-  if (data_hid < 0) 
+#if HAVE_HDF5_18
+  data_hid = H5Dcreate (group_hid, "m", H5T_NATIVE_UCHAR, space_hid,
+                        H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+#else
+  data_hid = H5Dcreate (group_hid, "m", H5T_NATIVE_UCHAR, space_hid,
+                        H5P_DEFAULT);
+#endif
+  if (data_hid < 0)
     {
       H5Sclose (space_hid);
       H5Gclose (group_hid);
       return false;
     }
-  
+
   tmp = m ();
   retval = H5Dwrite (data_hid, H5T_NATIVE_UCHAR, H5S_ALL, H5S_ALL, H5P_DEFAULT,
-		     (void*) &tmp) >= 0;
+                     (void*) &tmp) >= 0;
   H5Dclose (data_hid);
   if (!retval)
-    {
-      H5Sclose (space_hid);
-      H5Gclose (group_hid);
-      return false;
-    }    
-
-  data_hid = H5Dcreate (group_hid, "prim", H5T_NATIVE_UINT, space_hid, 
-			H5P_DEFAULT);
-  if (data_hid < 0) 
     {
       H5Sclose (space_hid);
       H5Gclose (group_hid);
       return false;
     }
-  
+
+#if HAVE_HDF5_18
+  data_hid = H5Dcreate (group_hid, "prim", H5T_NATIVE_UINT, space_hid,
+                        H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+#else
+  data_hid = H5Dcreate (group_hid, "prim", H5T_NATIVE_UINT, space_hid,
+                        H5P_DEFAULT);
+#endif
+  if (data_hid < 0)
+    {
+      H5Sclose (space_hid);
+      H5Gclose (group_hid);
+      return false;
+    }
+
   itmp = primpoly ();
   retval = H5Dwrite (data_hid, H5T_NATIVE_UINT, H5S_ALL, H5S_ALL, H5P_DEFAULT,
-		     (void*) &itmp) >= 0;
+                     (void*) &itmp) >= 0;
   H5Dclose (data_hid);
   if (!retval)
     {
       H5Sclose (space_hid);
       H5Gclose (group_hid);
       return false;
-    }    
+    }
   H5Sclose (space_hid);
 
   // Octave uses column-major, while HDF5 uses row-major ordering
-  for (int i = 0, j = d.length() - 1; i < d.length (); i++, j--)
+  for (int i = 0, j = d.length () - 1; i < d.length (); i++, j--)
     hdims[i] = d (j);
 
-  space_hid = H5Screate_simple (d.length(), hdims, (hsize_t*) 0);
+  space_hid = H5Screate_simple (d.length (), hdims, (hsize_t*) 0);
   if (space_hid < 0) return false;
 
   double *mtmp = mval.fortran_vec ();
@@ -628,23 +667,28 @@ octave_galois::save_hdf5 (hid_t loc_id, const char *name, bool save_as_floats)
       save_type_hid = H5T_NATIVE_UCHAR;
       char *wtmp = (char *)vtmp;
       for (int i = 0; i < d.numel (); i++)
-	wtmp[i] = (char) mtmp[i];
+        wtmp[i] = (char) mtmp[i];
     }
   else if (tmp <= 16)
     {
       save_type_hid = H5T_NATIVE_USHORT;
       uint16_t *wtmp = (uint16_t *)vtmp;
       for (int i = 0; i < d.numel (); i++)
-	wtmp[i] = (uint16_t) mtmp[i];
+        wtmp[i] = (uint16_t) mtmp[i];
     }
   else
     {
       for (int i = 0; i < d.numel (); i++)
-	vtmp[i] = (uint32_t) mtmp[i];
+        vtmp[i] = (uint32_t) mtmp[i];
     }
 
-  data_hid = H5Dcreate (group_hid, "val", save_type_hid, space_hid, 
-			H5P_DEFAULT);
+#if HAVE_HDF5_18
+  data_hid = H5Dcreate (group_hid, "val", save_type_hid, space_hid,
+                        H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+#else
+  data_hid = H5Dcreate (group_hid, "val", save_type_hid, space_hid,
+                        H5P_DEFAULT);
+#endif
   if (data_hid < 0)
     {
       H5Sclose (space_hid);
@@ -653,68 +697,93 @@ octave_galois::save_hdf5 (hid_t loc_id, const char *name, bool save_as_floats)
     }
 
   retval = H5Dwrite (data_hid, save_type_hid, H5S_ALL, H5S_ALL,
-		     H5P_DEFAULT, (void*) vtmp) >= 0;
+                     H5P_DEFAULT, (void*) vtmp) >= 0;
 
   H5Dclose (data_hid);
   H5Sclose (space_hid);
   H5Gclose (group_hid);
+
+#elif defined (HAVE_OCTAVE_BASE_VALUE_GRIPE_LOAD_SAVE)
+  gripe_save ("hdf5");
+#else
+  warning ("galois: saving hdf5 files not available");
+#endif
+
   return retval;
 }
 
 bool
-octave_galois::load_hdf5 (hid_t loc_id, const char *name,
-			  bool have_h5giterate_bug)
+octave_galois::load_hdf5 (octave_hdf5_id loc_id, const char *name)
 {
   bool retval = false;
+
+#if defined (HAVE_HDF5)
+
   char mord;
   unsigned int prim;
   hid_t group_hid, data_hid, space_id;
   hsize_t rank;
 
+#if HAVE_HDF5_18
+  group_hid = H5Gopen (loc_id, name, H5P_DEFAULT);
+#else
   group_hid = H5Gopen (loc_id, name);
+#endif
   if (group_hid < 0 ) return false;
 
+#if HAVE_HDF5_18
+  data_hid = H5Dopen (group_hid, "m", H5P_DEFAULT);
+#else
   data_hid = H5Dopen (group_hid, "m");
+#endif
   space_id = H5Dget_space (data_hid);
   rank = H5Sget_simple_extent_ndims (space_id);
 
   if (rank != 0)
-    { 
+    {
       H5Dclose (data_hid);
       H5Gclose (group_hid);
       return false;
     }
 
-  if (H5Dread (data_hid, H5T_NATIVE_UCHAR, H5S_ALL, H5S_ALL, 
-	       H5P_DEFAULT, (void *) &mord) < 0)
-    { 
+  if (H5Dread (data_hid, H5T_NATIVE_UCHAR, H5S_ALL, H5S_ALL,
+               H5P_DEFAULT, (void *) &mord) < 0)
+    {
       H5Dclose (data_hid);
       H5Gclose (group_hid);
       return false;
     }
 
   H5Dclose (data_hid);
+#if HAVE_HDF5_18
+  data_hid = H5Dopen (group_hid, "prim", H5P_DEFAULT);
+#else
   data_hid = H5Dopen (group_hid, "prim");
+#endif
   space_id = H5Dget_space (data_hid);
   rank = H5Sget_simple_extent_ndims (space_id);
 
   if (rank != 0)
-    { 
+    {
       H5Dclose (data_hid);
       H5Gclose (group_hid);
       return false;
     }
 
-  if (H5Dread (data_hid, H5T_NATIVE_UINT, H5S_ALL, H5S_ALL, 
-	       H5P_DEFAULT, (void *) &prim) < 0)
-    { 
+  if (H5Dread (data_hid, H5T_NATIVE_UINT, H5S_ALL, H5S_ALL,
+               H5P_DEFAULT, (void *) &prim) < 0)
+    {
       H5Dclose (data_hid);
       H5Gclose (group_hid);
       return false;
     }
 
   H5Dclose (data_hid);
+#if HAVE_HDF5_18
+  data_hid = H5Dopen (group_hid, "val", H5P_DEFAULT);
+#else
   data_hid = H5Dopen (group_hid, "val");
+#endif
   space_id = H5Dget_space (data_hid);
   rank = H5Sget_simple_extent_ndims (space_id);
 
@@ -731,11 +800,11 @@ octave_galois::load_hdf5 (hid_t loc_id, const char *name,
   OCTAVE_LOCAL_BUFFER (hsize_t, maxdims, rank);
 
   H5Sget_simple_extent_dims (space_id, hdims, maxdims);
-  MArray2<int> m (hdims[1], hdims[0]);
+  MArray<int> m (dim_vector (hdims[1], hdims[0]));
 
   int *re = m.fortran_vec ();
-  if (H5Dread (data_hid, H5T_NATIVE_UINT, H5S_ALL, H5S_ALL, 
-	       H5P_DEFAULT, (void *) re) >= 0) 
+  if (H5Dread (data_hid, H5T_NATIVE_UINT, H5S_ALL, H5S_ALL,
+               H5P_DEFAULT, (void *) re) >= 0)
     {
       retval = true;
       gval = galois (m, mord, prim);
@@ -743,12 +812,18 @@ octave_galois::load_hdf5 (hid_t loc_id, const char *name,
 
   H5Sclose (space_id);
   H5Dclose (data_hid);
-  return retval;
-}
+
+#elif defined (HAVE_OCTAVE_BASE_VALUE_GRIPE_LOAD_SAVE)
+  gripe_load ("hdf5");
+#else
+  warning ("galois: loading hdf5 files not available");
 #endif
 
+  return retval;
+}
+
 /*
-;;; Local Variables: ***
-;;; mode: C++ ***
-;;; End: ***
+  ;;; Local Variables: ***
+  ;;; mode: C++ ***
+  ;;; End: ***
 */
